#region Using

using System;
using System.Xml;
using System.Net;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Globalization;
using BlogEngine.Core.DataStore;

#endregion

public partial class widgets_Twitter_widget : WidgetBase
{

	private static DateTime LAST_MODIFIED = DateTime.MinValue;
	private const string CACHE_KEY = "twits";
	private static int MAX_ITEMS = 3;

	#region WidgetBase implementation

	public override string Name
	{
		get { return "Twitter"; }
	}

	public override bool IsEditable
	{
		get { return true; }
	}

	public override void LoadWidget()
	{
		StringDictionary settings = (StringDictionary)GetSettings(ObjectType.StringDictionary);
		hlTwitterAccount.NavigateUrl = settings["accounturl"];
		int.TryParse(settings["maxitems"], out MAX_ITEMS);

		if (settings.ContainsKey("feedurl"))
		{			
			if (HttpRuntime.Cache[CACHE_KEY] != null)
			{
				string xml = (string)HttpRuntime.Cache[CACHE_KEY];
				XmlDocument doc = new XmlDocument();
				doc.LoadXml(xml);
				BindFeed(doc, MAX_ITEMS);
			}

			if (DateTime.Now > LAST_MODIFIED.AddMinutes(30))
			{
				LAST_MODIFIED = DateTime.Now;
				BeginGetFeed(settings["feedurl"]);
			}
		}
	}

	#endregion

	#region Web methods

	private void BeginGetFeed(string url)
	{
		HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(url);
		request.Credentials = CredentialCache.DefaultNetworkCredentials;
		request.BeginGetResponse(EndGetResponse, request);
	}

	private void EndGetResponse(IAsyncResult result)
	{
		HttpWebRequest request = (HttpWebRequest)result.AsyncState;
		using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
		{
			XmlDocument doc = new XmlDocument();
			doc.Load(response.GetResponseStream());
			HttpRuntime.Cache[CACHE_KEY] = doc.OuterXml;
		}
	}

	#endregion

	#region Data binding

	private void BindFeed(XmlDocument doc, int maxItems)
	{
		XmlNodeList items = doc.SelectNodes("//channel/item");
		List<Twit> twits = new List<Twit>();
		for (int i = 0; i < maxItems; i++)
		{
			XmlNode node = items[i];
			
			string title = node.SelectSingleNode("description").InnerText;
			if (title.Contains(":"))
			{
				int start = title.IndexOf(":") + 1;
				title = title.Substring(start);
			}

			Twit twit = new Twit();
			twit.Title = title;
			twit.PubDate = DateTime.Parse(node.SelectSingleNode("pubDate").InnerText, CultureInfo.InvariantCulture);
			twit.Url = new Uri(node.SelectSingleNode("link").InnerText, UriKind.Absolute);

			twits.Add(twit);
		}
		
		twits.Sort();
		repItems.DataSource = twits;
		repItems.DataBind();
	}

	protected void repItems_ItemDataBound(object sender, RepeaterItemEventArgs e)
	{
		if (e.Item.ItemType == ListItemType.Item || e.Item.ItemType == ListItemType.AlternatingItem)
		{
			Label text = (Label)e.Item.FindControl("lblItem");
			Label date = (Label)e.Item.FindControl("lblDate");
			Twit twit = (Twit)e.Item.DataItem;
			text.Text = twit.Title;
			date.Text = twit.PubDate.ToString("MMMM d. HH:mm");
		}
	}

	#endregion

	private struct Twit : IComparable<Twit>
	{
		public string Title;
		public Uri Url;
		public DateTime PubDate;

		public int CompareTo(Twit other)
		{
			return other.PubDate.CompareTo(this.PubDate);
		}
	}
}
